"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AnalyticsServiceWorker = void 0;
const configuration_1 = require("@jfrog-pipelines/configuration");
const node_uuid_1 = __importDefault(require("node-uuid"));
const logger_1 = require("@jfrog-pipelines/logger");
const underscore_1 = require("underscore");
const Adapter_1 = __importDefault(require("./Adapter"));
const StepDetailsRepository_1 = require("./dao/StepDetailsRepository");
const TEST_SUITES_LIMIT = 100;
const PASSED = 'passed';
const FAILED = 'failures';
const ERRORS = 'errors';
const SKIPPED = 'skipped';
const limits = {
    passed: 20,
    failures: 40,
    error: 20,
    skipped: 20
};
class AnalyticsServiceWorker {
    constructor() {
        this.logger = logger_1.LoggerFactory.getLogger('AnalyticsServiceWorker');
    }
    async process(message) {
        try {
            if (typeof message.payload === 'object' && message.payload !== null) {
                const { stepId, stepletId } = message.payload;
                if (stepId && stepletId) {
                    this.logger.debug(`stepId received: `, stepId);
                    this.logger.debug(`stepletId received: `, stepletId);
                    const adapter = new Adapter_1.default(node_uuid_1.default.v4(), configuration_1.ConfigFactory.config.apiToken);
                    adapter.getTestReports(stepletId, (error, report) => {
                        this.processReports(report, stepId, stepletId);
                    });
                }
                else {
                    this.logger.error('stepId/stepletId is missing. Hence, cannot gather insight');
                }
            }
            await Promise.resolve();
        }
        catch (error) {
            this.logger.error('Error fetching test reports: ', error);
        }
    }
    async processReports(testReport, stepId, stepletId) {
        const suiteList = [];
        const logData = {
            stepletId: stepletId,
            total: 0,
            passed: 0,
            failed: 0,
            skipped: 0,
            errors: 0,
            duration: 0,
            testSuites: suiteList,
            timestamp: Date.now(),
            testSuiteCollection: '',
            stepId: stepId,
            pipelineName: '',
            stepName: '',
            runNumber: '',
            branchName: '',
            projectKey: ''
        };
        try {
            for (const report of testReport) {
                this.logger.info('$$ Inside forEach for testReport');
                logData.total += report.total ?? 0;
                logData.passed += report.totalPassing ?? 0;
                logData.failed += report.totalFailures ?? 0;
                logData.errors += report.totalErrors ?? 0;
                logData.skipped += report.totalSkipped ?? 0;
                logData.duration += report.durationSeconds ?? 0;
                logData.testSuites.push(this.getSuiteDetails(report));
            }
            this.logger.info('$$ logData: ', JSON.stringify(logData));
            const stepMetadata = await this.getStepMetadata(stepId);
            if (!stepMetadata) {
                this.logger.debug('Insight data incomplete, hence skipping to log data');
            }
            else {
                this.logInsightData(logData, stepMetadata);
            }
        }
        catch (error) {
            this.logger.warn('Error processing test reports: ', error);
        }
    }
    logInsightData(logData, stepMetadata) {
        try {
            const finalLogData = (0, underscore_1.extend)({}, logData, stepMetadata);
            this.logger.debug('Logging test insight: ', JSON.stringify(finalLogData));
            logger_1.LoggerFactory.logInsight(finalLogData);
        }
        catch (error) {
            this.logger.warn('Failed to log insight data: ', error);
        }
    }
    getSuiteDetails(report) {
        let suiteDetails = {};
        suiteDetails = this.initialiseSuiteDetails(suiteDetails, report);
        try {
            const suitesCapacityFull = function (suiteDetails) {
                return ((suiteDetails.passed?.length ?? 0) +
                    (suiteDetails.failures?.length ?? 0) +
                    (suiteDetails.errors?.length ?? 0) +
                    (suiteDetails.skipped?.length ?? 0) ===
                    TEST_SUITES_LIMIT);
            };
            const canSuspendLoop = function (counter, report, suiteDetails) {
                return (counter === TEST_SUITES_LIMIT ||
                    counter === (report.testElements?.length && report.testElements.length - 1) ||
                    suitesCapacityFull(suiteDetails));
            };
            if (report.testElements) {
                for (const [index, testElement] of report.testElements.entries()) {
                    const category = this.getResultCategory(testElement);
                    if (category === '' || (suiteDetails[category] && suiteDetails[category].length == limits[category])) {
                        continue;
                    }
                    else {
                        this.captureTestDetails(category, suiteDetails, testElement);
                    }
                    if (canSuspendLoop(index, report, suiteDetails)) {
                        if (suiteDetails.slowestTests && suiteDetails.slowestTests.length > 0) {
                            suiteDetails.slowestTests = this.sortSlowestTests(suiteDetails.slowestTests);
                        }
                        break;
                    }
                }
            }
        }
        catch (error) {
            this.logger.warn('Error reading test report: ', error);
        }
        return suiteDetails;
    }
    captureTestDetails(category, suiteDetails, testElement) {
        suiteDetails[category]?.push(testElement.name);
        const testName = {
            name: testElement.name,
            duration: testElement.durationSeconds ?? 0
        };
        suiteDetails.slowestTests?.push(testName);
    }
    initialiseSuiteDetails(suiteDetails, report) {
        suiteDetails.name = report.name;
        suiteDetails.totalTests = report.totalTests;
        suiteDetails.passed = [];
        suiteDetails.failures = [];
        suiteDetails.skipped = [];
        suiteDetails.errors = [];
        suiteDetails.duration = report.durationSeconds;
        suiteDetails.slowestTests = [];
        return suiteDetails;
    }
    sortSlowestTests(slowestTests) {
        return slowestTests.sort((a, b) => (b.duration ?? 0) - (a.duration ?? 0));
    }
    getResultCategory(result) {
        this.logger.info('In getResultCategory, received input: ', JSON.stringify(result));
        const map = {
            totalPassing: PASSED,
            totalFailures: FAILED,
            totalErrors: ERRORS,
            totalSkipped: SKIPPED
        };
        for (const key in result) {
            if (key in map && result[key]) {
                return map[key];
            }
        }
        return '';
    }
    async getStepMetadata(stepId) {
        try {
            const stepDetailsRepository = new StepDetailsRepository_1.StepDetailsRepository();
            const result = await stepDetailsRepository.getStepDetails(stepId);
            const resultObject = result?.[0] ?? [];
            const data = {};
            data.runNumber = resultObject.runNumber;
            data.branchName = resultObject.pipelineSourceBranch ?? '';
            data.pipelineName = resultObject.pipelinename ?? '';
            data.stepName = resultObject.stepname ?? '';
            data.projectKey = resultObject.projectname ?? '';
            let hasInvalidData = false;
            for (const [key, value] of Object.entries(data)) {
                if (key !== 'branchName' && !value) {
                    hasInvalidData = true;
                    break;
                }
            }
            return !hasInvalidData && data;
        }
        catch (error) {
            this.logger.warn('Failed to retrieve step data: ', error);
        }
    }
}
exports.AnalyticsServiceWorker = AnalyticsServiceWorker;
//# sourceMappingURL=worker.js.map